@@ -98,6 +98,9 @@
  * Default: false */
 #define VOIKKO_OPT_OCR_SUGGESTIONS 8
 
+/* (Spell checking only): Ignore non-words such as URLs and email addresses.
+ * Default: true */
+#define VOIKKO_OPT_IGNORE_NONWORDS 10
 
 /* Integer options */
 
@@ -244,6 +247,57 @@ char * voikko_hyphenate_cstr(int handle, const char * word);
  */
 char * voikko_hyphenate_ucs4(int handle, const wchar_t * word);
 
+/**
+ * Frees the memory allocated for spelling suggestions.
+ * @param suggest_result spelling suggestions
+ */
+void voikko_free_suggest_ucs4(wchar_t ** suggest_result);
+
+/**
+ * Frees the memory allocated for spelling suggestions.
+ * @param suggest_result spelling suggestions
+ */
+void voikko_free_suggest_cstr(char ** suggest_result);
+
+/**
+ * Frees the memory allocated for hyphenation results.
+ * @param hyphenate_result hyphenation result
+ */
+void voikko_free_hyphenate(char * hyphenate_result);
+
+/**
+ * Token types for string tokenization
+ * TOKEN_NONE:        End of text or error
+ * TOKEN_WORD:        Word
+ * TOKEN_PUNCTUATION: Punctuation
+ * TOKEN_WHITESPACE:  Whitespace
+ * TOKEN_UNKNOWN:     Character not used in Finnish
+ */
+enum voikko_token_type {TOKEN_NONE, TOKEN_WORD, TOKEN_PUNCTUATION, TOKEN_WHITESPACE, TOKEN_UNKNOWN};
+
+/**
+ * Find the next token in text stream.
+ * @param handle voikko instance
+ * @param text Pointer to the start of a text buffer
+ * @param textlen Number of characters left in the buffer
+ * @param tokenlen (out) Number of characters in the identified token
+ * @return Type of the identified token.
+ */
+enum voikko_token_type voikko_next_token_ucs4(int handle, const wchar_t * text, size_t textlen,
+                                              size_t * tokenlen);
+
+/**
+ * Find the next token in text stream.
+ * @param handle voikko instance
+ * @param text Pointer to the start of a text buffer
+ * @param textlen Number of bytes left in the buffer
+ * @param tokenlen (out) Number of characters in the identified token
+ * @return Type of the identified token.
+ */
+enum voikko_token_type voikko_next_token_cstr(int handle, const char * text, size_t textlen,
+                                              size_t * tokenlen);
+
+
 END_C_DECLS
 #endif
 
