@@ -65,6 +65,10 @@ int voikko_set_bool_option(int handle, int option, int value) {
 			if (value) voikko_options.suggestion_type = ST_OCR;
 			else voikko_options.suggestion_type = ST_STD;
 			return 1;
+		case VOIKKO_OPT_IGNORE_NONWORDS:
+			if (value) voikko_options.ignore_nonwords = 1;
+			else voikko_options.ignore_nonwords = 0;
+			return 1;
 	}
 	return 0;
 }
@@ -116,6 +120,7 @@ const char * voikko_init_with_path(int * handle, const char * langcode,
 	voikko_options.ignore_dot = 0;
 	voikko_options.ignore_numbers = 0;
 	voikko_options.ignore_uppercase = 0;
+	voikko_options.ignore_nonwords = 1;
 	voikko_options.accept_first_uppercase = 1;
 	voikko_options.accept_all_uppercase = 1;
 	voikko_options.no_ugly_hyphenation = 0;
@@ -234,6 +239,7 @@ int voikko_find_malaga_project(char * buffer, size_t buflen, const char * langco
 	DWORD dwBufLen;
 	LONG lRet;
 #endif // WIN32
+	char * path_from_env;
 	// Minimum sensible size for the buffer
 	if (buflen < 18) return 0;
 	char * tmp_buf = malloc(buflen + 2048);
@@ -252,6 +258,19 @@ int voikko_find_malaga_project(char * buffer, size_t buflen, const char * langco
 				return 1;
 			}
 		}
+
+		/* Check the path specified by environment variable VOIKKO_DICTIONARY_PATH */
+		/* FIXME: thread safety */
+		path_from_env = getenv("VOIKKO_DICTIONARY_PATH");
+		if (path_from_env && strlen(path_from_env) < buflen - 18) {
+			strcpy(buffer, path_from_env);
+			strcpy(buffer + strlen(path_from_env), "/" VOIKKO_DICTIONARY_FILE);
+			if (voikko_check_file(buffer)) {
+				free(tmp_buf);
+				return 1;
+			}
+		}
+
 		#ifdef HAVE_GETPWUID_R
 		/* Check for project file in $HOME/.voikko/VOIKKO_DICTIONARY_FILE */
 		getpwuid_r(getuid(), &pwd, tmp_buf, buflen + 2048, &pwd_result);
@@ -337,3 +356,21 @@ int voikko_check_file(const char * name) {
  #endif // WIN32
 #endif // HAVE_GETPWUID_R
 }
+
+void voikko_free_suggest_ucs4(wchar_t ** suggest_result) {
+	if (suggest_result) {
+		for (wchar_t ** p = suggest_result; *p; p++) free(*p);
+		free(suggest_result);
+	}
+}
+
+void voikko_free_suggest_cstr(char ** suggest_result) {
+	if (suggest_result) {
+		for (char ** p = suggest_result; *p; p++) free(*p);
+		free(suggest_result);
+	}
+}
+
+void voikko_free_hyphenate(char * hyphenate_result) {
+	free(hyphenate_result);
+}
